/* $Workfile: OtxHdlc.h $   $Revision: 28 $    $Modtime: 8/06/04 2:59p $  */
#ifndef OTX_HDLC_H
#define OTX_HDLC_H
/*******************************************************************************

  @doc PublicHwApi OtxHdlc

  Copyright (C) 1998 - 2003 Odin TeleSystems Inc., All Rights Reserved

  OTX Generic Hardware Driver API

  @module
  OtxHdlc | Prototypes for the Logical HDLC Device (Sender/Receiver) Functions

  @contents2
  OtxHdlc.h |

  @index func | OtxHdlc

  @contents2
  OTX Logical Hdlc Sender/Receiver Functions - OtxHdlcXXX() | The API functions for the Logical High-level Data Link Controller (HDLC) device

  @head3 OtxHdlcXXX() Function Reference |

  @index func | OtxHdlc

*******************************************************************************/

#ifdef __cplusplus
  extern "C" {
#endif

#include "OtxType.h"
#include "OtxDef.h"

#include "OtxHdlcD.h"


/*******************************************************************************

  @func
    Initializate the HDLC Sender.

  @comm
    Start HDLC operation with by calling the function <f OtxDrvEnable> and stop
    it by calling the function <f OtxDrvDisable>.

    For a description of how to connect of pipe to HDLC of various capacities 
    (64kpbs, 56kpbs, 16kpbs, Nx64kpbs, etc) please refer to <f OtxDrvConnectPipe>.

  @ex |
    // two HDLC logical devices in this example, logical devices can be hosted
    // either in Line Interface device (Li) or DSP

    for (i = 0; i < 2; i++) {
      // create logical devices
      result = OtxDrvCreateLogicalDevice(
	                g_hPhysDevs.hHostDevice[i],
	                OTX_LDEVICE_HDLC_SENDER,
	                g_hEventQueue,
	                i+100,
	                &g_sqHdlcSender[i].hHandle);

      // pipe creation and connection, see example on 
      @ex <f OtxDrvConnectPipe> |

      // connect logical devices
      result = OtxDrvConnectLogicalDevice(
	                g_sqHdlcSender[i].hHandle,
	                -1,
	                g_sqHdlcSender[i].hPipe,
	                -1);

      // initialize the HDLC Sender
      result = OtxHdlcSenderInit(
	                g_hLogicDevs.hHdlcSender,
	                OTX_HDLC_FFT_FLAGS,
	                2,
	                OTX_HDLC_CRC16,
	                OTX_FALSE,
	                OTX_TRUE,
	                OTX_HDLC_TR_NA);

      // enable HDLC sender before it can send frame
      result = OtxDrvEnable(
	                g_hLogicDevs.hHdlcSender,
	                OTX_TASK_SYNC,
	                5000L);
    }

  @xref
    <f OtxDrvConnectPipe>
    <nl><f OtxHdlcReceiverInit>
    <nl><t OtxHdlcFrameFillTypeE>
    <nl><t OtxHdlcCrcModeE>
    <nl><t OtxHdlcTxRateE>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxHdlcSenderInit(
  IN OTX_HANDLE       hHdlcDevice,            // @parm Handle to logical HDLC Filter Device
  IN OtxHdlcFrameFillTypeE  eFrameFillType,   // @parm Fill character(s) to be send between HDLC frames. See also <t OtxHdlcFrameFillTypeE>
  IN OTX_INT32        nInterFrameTimeFill,    // @parm Minimum No of interframe time characters between 2 HDLC frames (1=shared flags, 2=non-shared) Min value is 1. Max is 1024.
  IN OtxHdlcCrcModeE  eCrcMode,               // @parm Cyclic Redundancy Check (CRC) mode: 16-bit, 32-bit, or none. See also <t OtxHdlcCrcModeE>
  IN OTX_BOOL         bBitInversion,          // @parm If OTX_TRUE then all bits are inverted when transmitted and before received
  IN OTX_BOOL         bAppendTxCrc,           // @parm If OTX_TRUE the HDLC device will append the CRC to the frame to be transmitted. If OTX_FALSE, the CRC is generated with software
  IN OtxHdlcTxRateE   eTxRate                 // @parm HDLC Transmission Rate for V.110 and X.30 modes. See also <t OtxHdlcTxRateE>
);


/*******************************************************************************

  @func
    Initializate the HDLC Receiver.

  @comm
    Start HDLC operation with by calling the function <f OtxDrvEnable> and stop
    it by calling the function <f OtxDrvDisable>
  
    For a description of how to connect of pipe to HDLC of various capacities 
    (64kpbs, 56kpbs, 16kpbs, Nx64kpbs, etc) please refer to <f OtxDrvConnectPipe>.

  @ex |
    // two HDLC logical devices in this example, logical devices can be hosted
    // either in Line Interface device (Li) or DSP

    for (i = 0; i < 2; i++) {
      // create logical devices
      result = OtxDrvCreateLogicalDevice(
	                g_hPhysDevs.hHostDevice[i],
	                OTX_LDEVICE_HDLC_RECEIVER,
	                g_hEventQueue,
	                i+100,
	                &g_sqHdlcReceiver[i].hHandle);

      // pipe creation and connection, see example on 
      @ex <f OtxDrvConnectPipe> |

      // connect logical devices
      result = OtxDrvConnectLogicalDevice(
	                g_sqHdlcReceiver[i].hPipe,
	                -1,
	                g_sqHdlcReceiver[i].hHandle,
	                -1);

      // initialize the HDLC Receiver
      result = OtxHdlcReceiverInit(
	                g_sqHdlcReceiver[i].hHandle,
	                OTX_HDLC_FFT_FLAGS,
	                2,
	                OTX_HDLC_CRC16,
	                OTX_FALSE,
	                OTX_TRUE,
	                OTX_HDLC_TR_NA,
	                OTX_FALSE);

      // enable HDLC sender before it can receive frame
      result = OtxDrvEnable(
	                g_sqLiHdlcReceiver[i].hHandle,
	                OTX_TASK_SYNC,
	                5000L);

    }

  @xref
    <f OtxDrvConnectPipe>
    <nl><f OtxHdlcSenderInit>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_HDLC_SENDER>
    <nl><t OtxHdlcFrameFillTypeE>
    <nl><t OtxHdlcCrcModeE>
    <nl><t OtxHdlcTxRateE>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxHdlcReceiverInit(
  IN OTX_HANDLE       hHdlcDevice,            // @parm Handle to logical HDLC Filter Device
  IN OtxHdlcFrameFillTypeE  eFrameFillType,   // @parm Fill character(s) to be send between HDLC frames. See also <t OtxHdlcFrameFillTypeE>
  IN OTX_INT32        nInterFrameTimeFill,    // @parm Minimum No of interframe time characters between 2 HDLC frames (1=shared flags, 2=non-shared) Min value is 1. Max is 1024.
  IN OtxHdlcCrcModeE  eCrcMode,               // @parm Cyclic Redundancy Check (CRC) mode: 16-bit, 32-bit, or none. See also <t OtxHdlcCrcModeE>
  IN OTX_BOOL         bBitInversion,          // @parm If OTX_TRUE then all bits are inverted when transmitted and before received
  IN OTX_BOOL         bExcludeRxCrc,          // @parm If OTX_TRUE the HDLC device will remove the CRC from the received frame, if OTX_FALSE the CRC is passed on to the receiving software
  IN OtxHdlcTxRateE   eTxRate,                // @parm HDLC Transmission Rate for V.110 and X.30 modes. See also <t OtxHdlcTxRateE>
  IN OTX_BOOL         bEventForEveryFrame     // @parm If OTX_TRUE the HDLC device will generate an event for every received frame, if OTX_FALSE then the HDLC device will only generate an event when one or more frames have been received since the last call to <f OtxHdlcReadData> or <f OtxHdlcReadDataEx>.
);


/*******************************************************************************

  @func
    Sends transparent data or HDLC frames on a configured HDLC pipe.

  @ex |
      // send one HDLC frame with data "00 01 02 03 04 05 06 07 08 09"
      for (i = 0; i < 10; i++) {
        arMsg[i] = (OTX_UINT8) i;
      }

      // send one HDLC Frame
      result = OtxHdlcSendData(
                  g_hLogicDevs.hHdlcSender[nCurrentLi], 
                  arMsg, 
                  10, 
                  DEMO_TASK_REF, 
                  OTX_TIME_INFINITY); 

  @xref
    <f OtxHdlcSS7SendData>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxHdlcSendData(
  IN OTX_HANDLE       hHdlcDevice,            // @parm Handle to the HDLC Sender Device
  IN OTX_UINT8        *sqData,                // @parm Data to be sent
  IN OTX_UINT32       nDataLen,               // @parm Length of the data to be send
  IN OTX_TASK_REF     nTaskRef,               // @parm See <t OTX_TASK_REF>
  IN OTX_TIME         nWaitMaxMs              // @parm See <t OTX_TIME>
);


/*******************************************************************************

  @func
    Flushes (empties) the queue of HDLC frames waiting to be transmitted (if any),
    
  @ex | 
      // from the following example, 10 HDLC frames are waiting to be transmitted, 
      // when the sleep time is short enough before all the 10 HDLC frames are 
      // transmitted, several (less than 10) HDLC frames can be detected, because 
      // others are removed from the waiting queue; if the sleep time is long enough,
      // 10 HDLC frames would be detected.
      
      hHandle = g_hLogicDevs.hHdlcSender;

      // data to be transmitted, length 100 
      for (i = 0; i < 100; i++) {
        arMsg[i] = (OTX_UINT8) i;
      }

      // Send the HDLC frame
      for (i = 0; i < 10; i++) {
        result = OtxHdlcSendData(
                    hHandle, 
                    arMsg, 
                    100, 
                    DEMO_TASK_REF, 
                    200); 
      }

      // Sleeping...
      OsaThreadSleep(50);

      // flush HDLC sender
      result = OtxHdlcSenderFlush(hHandle); 
  
  @xref
    <f OtxHdlcSendData>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxHdlcSenderFlush(
  IN OTX_HANDLE       hHdlcDevice             // @parm Handle to the HDLC Sender Device
);


/*******************************************************************************

  @func
    Read data received by the HDLC receiver

  @rdesc Returns one of the following values:

    @flag OTX_S_OK | Hdlc Data was fetched
    @flag OTX_S_FALSE | No Hdlc Data available at this moment
    @flag OTX_E_BUFFER_TO_SMALL | The supplied user data buffer is too small to fit the received HDLC frame

  @comm
    Usually it is used in driver event case "OTX_LDEVICE_HDLC_RECEIVER"  

  @ex |
    // Note: function DemoLogf() is used to display message in our demo programs 

    case OTX_LDEVICE_HDLC_RECEIVER:
      otxRc = OtxDrvEventCode2String(
                 pEventData->m_hDevice, 
                 pEventData->m_nCode, 
                 256, 
                 &msgStr[0]);
      
      if (OTX_SUCCESS(otxRc)) {
        // Event from HLDC Receiver 
        if (pEventData->m_nCode == OTX_HDLC_RECEIVER_EC_COMPLETE_FRAME) {
        
          otxRc = OtxHdlcReadData(
                     pEventData->m_hDevice, 
                     buf, 
                     1024, 
                     &nFrameLen, 
                     &nFrameSeq, 
                     &nFrameEndCode, 
                     &nTimeStamp);
          
          while (OTX_RESULT_CODE(otxRc) == OTX_S_OK) {

            if (pEventData->m_nSourceId < 256) {
              g_coHdlcRxFrames[pEventData->m_nSourceId]++;
            }

            if (g_bHdlcVerbose) {
              DemoLogf("HDLC Frame Received, Sequence Number = %d\n", pEventData->m_nParam);

              DemoLogf("HdlcReceiver[%d] Len=%d, Fe=0x%.2X, Sq=%d: ", pEventData->m_nSourceId, nFrameLen, nFrameEndCode, nFrameSeq);
              for (i = 0; i < nFrameLen && i < 1024; i++) {
                DemoLogf("%.2X ", buf[i]);
              }
              DemoLogf("\n");
            } else {
              if (nFrameEndCode) {

                g_coHdlcBadRxFrames[pEventData->m_nSourceId]++;
              
                DemoLogf("FrameEndCode Error HdlcReceiver[%d] Len=%d, Fe=0x%.2X, Sq=%d: ", pEventData->m_nSourceId, nFrameLen, nFrameEndCode, nFrameSeq);
                for (i = 0; i < nFrameLen && i < 256; i++) {
                  DemoLogf("%.2X ", buf[i]);
                }
                DemoLogf("\n");
              
                if (g_coHdlcBadRxFrames[pEventData->m_nSourceId] > 100) {
                  DemoLogf("Too many bad frames from HdlcReceiver[%d]. Disabling device\n", pEventData->m_nSourceId);
                  otxRc = OtxDrvDisable(
                            pEventData->m_hDevice,  
                            OTX_TASK_SYNC, 
                            5000L); 
                }

              } else {
                if ((nFrameLen != (OTX_UINT32)g_cbHdlcFrame) && (nFrameLen != 3)) {
                  g_coHdlcBadRxFrames[pEventData->m_nSourceId]++;
                
                  DemoLogf("LENGTH Error HdlcReceiver[%d] Len=%d, Fe=0x%.2X, Sq=%d: ", pEventData->m_nSourceId, nFrameLen, nFrameEndCode, nFrameSeq);
                  for (i = 0; i < nFrameLen && i < 256; i++) {
                    DemoLogf("%.2X ", buf[i]);
                  }
                  DemoLogf("\n");
                
                } else if (memcmp(buf+3, g_sqHdlcFrame+3, g_cbHdlcFrame-3)) {
                  g_coHdlcBadRxFrames[pEventData->m_nSourceId]++;
                
                  DemoLogf("memcmp Error HdlcReceiver[%d] Len=%d, Fe=0x%.2X, Sq=%d: ", pEventData->m_nSourceId, nFrameLen, nFrameEndCode, nFrameSeq);
                  for (i = 0; i < nFrameLen && i < 256; i++) {
                    DemoLogf("%.2X ", buf[i]);
                  }
                  DemoLogf("\n");                
                }
              }
            }
            otxRc = OtxHdlcReadData(
                       pEventData->m_hDevice, 
                       buf, 
                       1024, 
                       &nFrameLen, 
                       &nFrameSeq, 
                       &nFrameEndCode, 
                       &nTimeStamp);
          }
          if (OTX_RESULT_CODE(otxRc) != OTX_S_FALSE) {
            DemoLogf("HdlcReceiver[%d] OtxHdlcReadData() was unsuccessful (rc = %X).\n", pEventData->m_nSourceId, otxRc);
          }

        } else {
          // Unexpected event from this device
          DemoLogf("HdlcReceiver[%d] Event: %s \n", pEventData->m_nSourceId, msgStr);
        }
      } else {
        DemoLogf("HdlcReceiver[%d] Invalid Event: %ld \n", pEventData->m_nSourceId, pEventData->m_nCode);
      }
      break;  // end of OTX_LDEVICE_HDLC_RECEIVER

  @xref
    <f OtxHdlcReceiverInit>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_HDLC_RECEIVER>
    <nl><t OtxHdlcFrameEndCodeE>
    <nl><f OtxHdlcSendData>
    <nl><f OtxHdlcReadDataEx>
    
*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxHdlcReadData(
  IN  OTX_HANDLE      hHdlcDevice,            // @parm Handle to the HDLC Receiver Device
  OUT OTX_UINT8       *pDataBuf,              // @parm Data buffer for the received data
  IN  OTX_UINT32      nBufLen,                // @parm Length of the user data buffer
  OUT OTX_UINT32      *pnFrameLen,            // @parm Length of the received HDLC frame
  OUT OTX_UINT32      *pnFrameSeq,            // @parm Sequence number of the received HDLC frame (matches the number in the event queue)
  OUT OtxHdlcFrameEndCodeE  *pnFrameEndCode,  // @parm Frame Status Code
  OUT OTX_DATETIME    *pnTimeStamp            // @parm Timestamp of the last received octet of the frame
);


/*******************************************************************************

  @func
    Read multiple data frames received by the HDLC receiver.

  @rdesc Returns one of the following values:

    @flag OTX_S_OK | Hdlc Data was fetched
    @flag OTX_S_FALSE | No Hdlc Data available at this moment
    @flag OTX_E_BUFFER_TO_SMALL | The supplied user data buffer is too small to fit the first HDLC frame info and data 

  @comm
    The data in the returned buffer is organized so that it always starts with 
    a frame info struct: OtxHdlcFrameInfoS, after which we have the HDLC data, 
    the next OtxHdlcFrameInfoS follows shortly after the last data byte. 
    In order to find the next frame, all frames info structs (except the last
    one) contain a field with the relative index of the start of the next frame. 
    The index is a byte index relative to the start of the current frame's info 
    struct, i.e. if we have a pointer p1, to the beginning of the first info 
    struct in the buffer we get a pointer to the second struct p2, by adding 
    the byte offset found in the current info struct, i.e.:

    OtxHdlcFrameInfoS p1 = buffer;
    OtxHdlcFrameInfoS p2 = OTX_STRUCT_ADD_BYTE_OFFSET(OtxHdlcFrameInfoS, p1, p1->nOffsetNext);

    This dispenses with the need to worry about the byte alignments of the OtxHdlcFrameInfoS structs.

    The last frame's nOffsetNext field will be zero

  @ex | 

    // Note: function DemoLogf() is used to display message in our demo programs  

    // Multiple frame read
    case OTX_LDEVICE_HDLC_RECEIVER:
      // Event from HLDC Receiver 
      if (pEventData->m_nCode == OTX_HDLC_RECEIVER_EC_COMPLETE_FRAME) {
        OTX_UINT8  buf[CB_BUF_SIZE];
        OTX_UINT8  *pBuf;
			  OtxHdlcFrameInfoS *pFrame;
        OTX_UINT32 coFrames;
        OTX_UINT32 j;
      
        otxRc = OtxHdlcReadDataEx(
                   pEventData->m_hDevice, 
                   buf, 
                   CB_BUF_SIZE, 
                   DEMO_MAX_MULTIPLE_FRAME_READ, 
                   OTX_FALSE, 
                   &coFrames);
        
        while (OTX_RESULT_CODE(otxRc) == OTX_S_OK) {
      
          if (pEventData->m_nSourceId < 32) {
            g_coHdlcRxFrames[pEventData->m_nSourceId] += coFrames;
          }

          if (g_bHdlcVerbose) {
            DemoLogf("%d HDLC Frame(s) Received, Sequence Number = %d\n", coFrames, pEventData->m_nParam);
          }
        
          pFrame = (OtxHdlcFrameInfoS *) buf;

          for (j = 0; j < coFrames; j++) {

					  pBuf = pFrame->sqData;

            if (g_bHdlcVerbose) {
              DemoLogf("HdlcReceiver[%d] Len=%d, Fe=0x%.2X, Sq=%d: ", pEventData->m_nSourceId, pFrame->cbOctets, pFrame->nFrameEndCode, pFrame->nFrameSeq);
              for (i = 0; i < pFrame->cbOctets && i < CB_BUF_SIZE; i++) {
                DemoLogf("%.2X ", pBuf[i]);
              }
              DemoLogf("\n");
            } else {
              if (pFrame->nFrameEndCode) {
              
                g_coHdlcBadRxFrames[pEventData->m_nSourceId]++;
              
                DemoLogf("FrameEndCode Error HdlcReceiver[%d] Len=%d, Fe=0x%.2X, Sq=%d: ", pEventData->m_nSourceId, pFrame->cbOctets, pFrame->nFrameEndCode, pFrame->nFrameSeq);
                for (i = 0; i < pFrame->cbOctets && i < 256; i++) {
                  DemoLogf("%.2X ", pBuf[i]);
                }
                DemoLogf("\n");
              
                if (g_coHdlcBadRxFrames[pEventData->m_nSourceId] > 100) {
                  DemoLogf("Too many bad frames from HdlcReceiver[%d]. Disabling device\n", pEventData->m_nSourceId);
                  otxRc = OtxDrvDisable(
                            pEventData->m_hDevice,  
                            OTX_TASK_SYNC, 
                            5000L); 
                }                          
              } else {
                if ((pFrame->cbOctets != (OTX_UINT32)g_cbHdlcFrame) && (pFrame->cbOctets != 3)) {
                  g_coHdlcBadRxFrames[pEventData->m_nSourceId]++;
                
                  DemoLogf("LENGTH Error HdlcReceiver[%d] Len=%d, Fe=0x%.2X, Sq=%d: ", pEventData->m_nSourceId, pFrame->cbOctets, pFrame->nFrameEndCode, pFrame->nFrameSeq);
                  for (i = 0; i < pFrame->cbOctets && i < 256; i++) {
                    DemoLogf("%.2X ", pBuf[i]);
                  }
                  DemoLogf("\n");
                
                } else if (memcmp(pBuf+3, g_sqHdlcFrame+3, g_cbHdlcFrame-3)) {
                  g_coHdlcBadRxFrames[pEventData->m_nSourceId]++;
                
                  DemoLogf("memcmp Error HdlcReceiver[%d] Len=%d, Fe=0x%.2X, Sq=%d: ", pEventData->m_nSourceId, pFrame->cbOctets, pFrame->nFrameEndCode, pFrame->nFrameSeq);
                  for (i = 0; i < pFrame->cbOctets && i < 256; i++) {
                    DemoLogf("%.2X ", pBuf[i]);
                  }
                  DemoLogf("\n");                
                }
              }
            }  

            // Step pointer to next frame (if any)
					  pFrame = OTX_STRUCT_ADD_BYTE_OFFSET(OtxHdlcFrameInfoS, pFrame, pFrame->nOffsetNext);
          }

          // Read next frame (if any)
          otxRc = OtxHdlcReadDataEx(
                     pEventData->m_hDevice, 
                     buf, 
                     CB_BUF_SIZE, 
                     DEMO_MAX_MULTIPLE_FRAME_READ, 
                     OTX_FALSE, 
                     &coFrames);
        } 
        if (OTX_RESULT_CODE(otxRc) != OTX_S_FALSE) {
          DemoLogf("HdlcReceiver[%d] OtxHdlcReadDataEx() was unsuccessful.\n", pEventData->m_nSourceId);
          DemoCheckRc(pEventData->m_hDevice, otxRc);
        }            
      } else {
        otxRc = OtxDrvEventCode2String(
                   pEventData->m_hDevice, 
                   pEventData->m_nCode, 
                   256, 
                   &msgStr[0]);
        
        if (OTX_SUCCESS(otxRc)) {
          DemoLogf("HdlcReceiver[%d] Event: %s \n", pEventData->m_nSourceId, msgStr);
        } else {
          DemoLogf("HdlcReceiver[%d] Invalid Event: %ld \n", pEventData->m_nSourceId, pEventData->m_nCode);
        }
      }
      break; // end of OTX_LDEVICE_HDLC_RECEIVER

  @xref
    <f OtxHdlcReceiverInit>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_HDLC_RECEIVER>
    <nl><f OtxHdlcSendData>
    <nl><f OtxHdlcReadData>
    <nl><t OtxHdlcFrameInfoS>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxHdlcReadDataEx(             
  IN  OTX_HANDLE      hHdlcDevice,            // @parm Handle to the HDLC Receiver Device
  OUT OTX_UINT8       *psqInfoAndDataBuffer,  // @parm Data buffer for the received data and info for all received frames
  IN  OTX_UINT32      cbBufferCapacity,       // @parm Length of the buffer
  IN  OTX_UINT32      coMaxFrames,            // @parm Maximum number of HDLC frames to be read by this call. A count of zero means read until buffer full
  IN  OTX_BOOL        bTruncate,              // @parm Utilize whole buffer even if it means that the last frame is truncated and the remainder discarded
  OUT OTX_UINT32      *pcoFrames              // @parm Number of frames actually placed in buffer upon return
);


/*******************************************************************************

  @func
    Get the status and statistics of the incoming SS7 link on a specified pipe

  @ex |

      OTX_UINT32  nFisuCount;   // Number of FISUs received
      OTX_UINT32  nLssuCount;   // Number of LSSUs received
      OTX_UINT32  nMsuCount;    // Number of MSUs received
      OtxHdlcSS7FisuS fisu;     // The last received FISU on the pipe.

      result = OtxHdlcSS7GetReceiveStatus(
                  g_sqDspHdlcReceiver.hHandle, 
                  &nFisuCount, 
                  &nLssuCount, 
                  &nMsuCount, 
                  &fisu); 

      // print out status
      if (OTX_SUCCESS(result)) {
        DemoLogf("FISUs received: %lu\n", nFisuCount);
        DemoLogf("LSSUs received: %lu\n", nLssuCount);
        DemoLogf("MSUs  received: %lu\n", nMsuCount);
        DemoLogf("Last FISU received: bsn = %d, bib = %d, fsn = %d, fib = %d, 
                  li = %d\n", fisu.bsn, fisu.bib, fisu.fsn, fisu.fib, fisu.li);
      }    

  @xref
    <f OtxHdlcSS7SetFilter>
    <nl><f OtxHdlcSS7GetFilter>
    <nl><t OtxHdlcSS7FisuS>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxHdlcSS7GetReceiveStatus(
  IN  OTX_HANDLE      hHdlcDevice,            // @parm Handle to the HDLC Device
  OUT OTX_UINT32      *pnFisuCount,           // @parm Number of FISUs received
  OUT OTX_UINT32      *pnLssuCount,           // @parm Number of LSSUs received
  OUT OTX_UINT32      *pnMsuCount,            // @parm Number of MSUs received
  OUT OtxHdlcSS7FisuS *pFisu                  // @parm The last received FISU on the pipe. See also <t OtxHdlcSS7FisuS>
);


/*******************************************************************************

  @func
    Set the filter mask for the filtering out FISUs, LSSUs, and/or MSUs

  @comm 

    It should be used after HDLC receiver device is initialized

  @ex |

    // set filter mask on 2 DSP
    for (nDsp = 0; nDsp < 2; nDsp++) {
    
      // Initialize the Hdlc Receiver
      result = OtxHdlcReceiverInit(
                  g_hLogicDevs[nDsp].hReceiver,   
                  OTX_HDLC_FFT_FLAGS,             
                  1,                              
                  OTX_HDLC_CRC16,                 
                  OTX_FALSE,                      
                  OTX_FALSE,                      
                  OTX_HDLC_TR_NA,                 
                  OTX_TRUE                        
        ); 

      result = OtxHdlcSS7SetFilter(
                  g_hLogicDevs[nDsp].hReceiver, 
                  OTX_HDLC_SS7_FILTER_FISU | OTX_HDLC_SS7_FILTER_SHORT_LSSU | OTX_HDLC_SS7_FILTER_LONG_LSSU); 
    }

  @xref
    <f OtxHdlcSS7GetFilter>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxHdlcSS7SetFilter(
  IN OTX_HANDLE       hHdlcDevice,            // @parm Handle to the HDLC Pipe
  IN OTX_UINT32       filterMask              // @parm Filter Mask (see HDLC_SS7_FILTER_XXXX Macros)
);


/*******************************************************************************

  @func
    Returns the currently active filter mask for filtering out FISUs, LSSUs 
    and/or MSUs

  @ex |
      
      OTX_HANDLE hHandle;
      OTX_UINT32 filterMask;

      hHandle = g_sqDspHdlcReceiver.hHandle;

      // toggle OTX_HDLC_SS7_FILTER_FISU_ALL filter mask on/off 
      result = OtxHdlcSS7GetFilter(
                  hHandle, 
                  &filterMask); 
      
      if (filterMask & OTX_HDLC_SS7_FILTER_FISU_ALL) {
        filterMask &= ~OTX_HDLC_SS7_FILTER_FISU_ALL;
      } else {
        filterMask |= OTX_HDLC_SS7_FILTER_FISU_ALL;
      }

      // reset the filter mask
      result = OtxHdlcSS7SetFilter(
                  hHandle, 
                  filterMask); 

  @xref
    <f OtxHdlcSS7SetFilter>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxHdlcSS7GetFilter(
  IN  OTX_HANDLE      hHdlcDevice,            // @parm Handle to the HDLC Device
  OUT OTX_UINT32      *pFilterMask            // @parm Currently active Filter Mask (see HDLC_SS7_FILTER_XXXX Macros)
);


/*******************************************************************************

  @func
    Send a message (LSSU, MSU, or arbitrary data) on a SS7 pipe. After the 
    sending of the message has been completed, continues sending the new FISU 
    provided as a parameter to the function.

  @ex |
  
    OtxHdlcSS7FisuS msu;
    OtxHdlcSS7FisuS *pFisu;
    OTX_UINT8 sqHdlcFrame[300];

    memcpy(&msu, pFisu, OTX_HDLC_SS7_CB_FISU);        // Build the first 3 bytes of the MSU
    msu.li = 10 - OTX_HDLC_SS7_CB_FISU;               // Set the Length Indicator
    memcpy(sqHdlcFrame, &msu, OTX_HDLC_SS7_CB_FISU);  // Put the first 3 bytes of the MSU in the HDLC frame buffer
    for (i = OTX_HDLC_SS7_CB_FISU; i < 10; i++) {     // Add the payload the HDLC frame buffer
      sqHdlcFrame[i + OTX_HDLC_SS7_CB_FISU] = (OTX_UINT8) i;
    }

    // Send MSU Frame with length of 10
    result = OtxHdlcSS7SendData(
                g_sqDspHdlcSender.hHandle, 
                sqHdlcFrame, 
                10, 
                pFisu, 
                DEMO_TASK_REF, 
                OTX_TIME_INFINITY); 
    
    if (OTX_RESULT_CODE(result) == OTX_E_BUSY) {
      // Driver IO Request buffer is full. We need to retransmit this frame.
      i--;
      busyCount++;
    } else {
      // time to sleep (in ms) between transmitted HDLC Frames
      OsaThreadSleep(10);
    }

  @xref
    <f OtxHdlcSendData>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxHdlcSS7SendData(
  IN OTX_HANDLE       hHdlcDevice,            // @parm Handle to the Logical HDLC Device
  IN OTX_UINT8        *psqData,               // @parm Data (MSU, LSSU, or other arbitrary frame) to be sent
  IN OTX_UINT32       nDataLen,               // @parm Length of the data to be sent
  OtxHdlcSS7FisuS     *pFisu,                 // @parm Next FISU to be sent after the data
  IN OTX_TASK_REF     nTaskRef,               // @parm See <t OTX_TASK_REF>
  IN OTX_TIME         nWaitMaxMs              // @parm See <t OTX_TIME>
);


/*******************************************************************************

  @func
    Load an HDLC frame to be repeated for the HDLC pipe and begin transmission. 
    The Repeat Data is an HDLC frame which will be sent continuosly when there 
    is no other user data frames to be sent. User data can be sent with 
    <f OtxHdlcSendData> function on which case the user data frame in 
    interleaved between two consecutive Repeat Data frames. After the sending 
    of user data frame has been completed, the HDLC controller will resume the 
    sending of the Repeat Data frame.

    This function can be used to continously send SS7 FISUs or LSSUs.

  @ex |

      OTX_UINT8 *pRepeatFrame = (OTX_UINT8 *)"\x10\x11\x12\x13\x14";
      OTX_HANDLE hHandle;

      hHandle = g_sqDspHdlcSender.hHandle;

      // start Repeat Data Frames with length of 5 from HDLC Sender 
      result = OtxHdlcStartRepeatData(
                  hHandle, 
                  pRepeatFrame, 
                  5, 
                  DEMO_TASK_REF, 
                  OTX_TIME_INFINITY); 

  @xref
   <f OtxHdlcStopRepeatData>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxHdlcStartRepeatData(
  IN OTX_HANDLE       hHdlcDevice,            // @parm Handle to the HDLC Device
  IN OTX_UINT8        *psqData,               // @parm Repeat Data frame to be sent
  IN OTX_UINT32       nDataLen,               // @parm Length of the data frame to be sent
  IN OTX_TASK_REF     nTaskRef,               // @parm See <t OTX_TASK_REF>
  IN OTX_TIME         nWaitMaxMs              // @parm See <t OTX_TIME>
);


/*******************************************************************************

  @func
    Stop sending of Repeat Data frames on a specified HDLC pipe.

  @ex |

    OTX_HANDLE hHandle;

    hHandle = g_sqDspHdlcSender.hHandle;

    // stop Repeat Data Frames from HDLC Sender 
    result = OtxHdlcStopRepeatData(hHandle);

  @xref
   <f OtxHdlcStartRepeatData>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxHdlcStopRepeatData(
  IN OTX_HANDLE       hHdlcDevice             // @parm Handle to the HDLC Device
);



#ifdef __cplusplus
  } // extern "C"
#endif


/*******************************************************************************

  Copyright (C) Odin TeleSystems Inc., 1998 - 2003

  All rights reserved. No part of this computer program may be reproduced,
  stored in any retrieval system, or transmitted, in any form or by any
  means, electronic, mechanical, photocopying, recording, or otherwise,
  without the prior written permission of Odin TeleSystems Inc., or without
  a permission stipulated in the agreement/contract under which this computer
  program was supplied.

*******************************************************************************/
#endif // OTX_HDLC_H
